//
//  For more information, please see: http://software.sci.utah.edu
//
//  The MIT License
//
//  Copyright (c) 2009 Scientific Computing and Imaging Institute,
//  University of Utah.
//
//
//  Permission is hereby granted, free of charge, to any person obtaining a
//  copy of this software and associated documentation files (the "Software"),
//  to deal in the Software without restriction, including without limitation
//  the rights to use, copy, modify, merge, publish, distribute, sublicense,
//  and/or sell copies of the Software, and to permit persons to whom the
//  Software is furnished to do so, subject to the following conditions:
//
//  The above copyright notice and this permission notice shall be included
//  in all copies or substantial portions of the Software.
//
//  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
//  OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
//  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
//  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
//  DEALINGS IN THE SOFTWARE.
//
//    File   : BaseTool.cc
//    Author : Martin Cole
//    Date   : Thu May 25 21:08:59 2006

#include <Core/Events/BaseEvent.h>
#include <Core/Events/BaseTool.h>
#include <string>
#include <iostream>

namespace SCIRun {



BaseTool::BaseTool(const std::string& name) :
  ref_cnt(0),
  name_(name)
{
}


BaseTool::BaseTool() :
  ref_cnt(0),
  name_("")
{
}


BaseTool::~BaseTool()
{
}


PointerTool::PointerTool(const std::string& name) :
  BaseTool(name)
{
  if (name_ == "") name_ = name;
}


PointerTool::~PointerTool()
{
}


KeyTool::KeyTool(const std::string& name) :
  BaseTool(name)
{
  if (name_ == "") name_ = name;
}


KeyTool::~KeyTool()
{
}


WindowTool::WindowTool(const std::string& name) :
  BaseTool(name)
{
  if (name_ == "") name_ = name;
}


WindowTool::~WindowTool()
{
}


TMNotificationTool::TMNotificationTool(const std::string& name) :
  BaseTool(name)
{
  if (name_ == "") name_ = name;
}


TMNotificationTool::~TMNotificationTool()
{
}


CommandTool::CommandTool(const std::string& name) :
  BaseTool(name)
{
  if (name_ == "") name_ = name;
}


CommandTool::~CommandTool()
{
}


} // namespace SCIRun

